#ifndef __ASTODBC_H__
#define __ASTODBC_H__

#include <time.h>

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

/*******************************************************************************************
 * ODBC connection
 *******************************************************************************************/
/* Creates a new ODBC connection.
 *
 *      connString      - connection string that will be passed to SQLDriverConnect(). It
 *                        should include Driver=value; setting. See docs at
 *                        http://msdn.microsoft.com/library/default.asp?url=/library/en-us/odbc/htm/odbcconnecting_with_sqldriverconnect.asp
 *      poolable        - set to not 0 if you want this connection to be shared with other
 *                        similar ODBC connections your application may open.
 *      writeable       - set to 0 is your application is going to do only SELECTs from the
 *                        database. Helps with ODBC optimizations
 *      pconn           - the output argument where the connection pointer gets written in
 *                        case of successful return.
 *
 * Returns "" if connection successfully opened and a non empty error text in case of a failure.
 * This error text is valid only until the next call to ast_odbc_* functions. Do not call
 * free() on this result
 * */
const char*     ast_odbc_conn_new       ( const char* connString, short poolable, short writeable, void** pconn );

/* Returns text of the last error that happened within this connection
 *
 *      conn            - your connection
 *
 * Returns text of the last error that happened within this connection
 * This error text is valid only until the next call to ast_odbc_* functions.
 * */
const char*     ast_odbc_conn_error_text( void* conn );

/* Destroys the collection
 *
 *      conn            - your connection
 *
 * Returns NULL in case of success. Otherwise returns its argument
 * */
void*           ast_odbc_conn_delete    ( void* conn );

/*******************************************************************************************
 * ODBC query
 *******************************************************************************************/
/* Creates new ODBC statement
 *
 *      conn            - your connection
 *      queryString     - SQL query string (SELECT, INSERT, UPDATE etc)
 *      pconn           - the output argument where the query pointer gets written in
 *                        case of successful return.
 *
 * Returns "" if query initialized and a non empty error text in case of a failure
 * This error text is valid only until the next call to ast_odbc_* functions.
 * Do not call free() on this text.
 *
 * NOTE ,that the query does not have to be a record set generating SELECT query.
 * It can be an INSERT or UPDATE query as well. In this case functions like
 * ast_odbc_query_fetch, ast_odbc_query_next, ast_odbc_query_eof, ast_odb_query_get*
 * will return an error.
 *
 * */
const char*     ast_odbc_query_new       ( void* conn, const char* queryString, void** pquery );

/* Returns !=0 if the result set enumerated by ast_odbc_query_fext or ast_odbc_query_next
 * has reached its end
 *
 *      query           - the query object returned from ast_odbc_query_new
 * */
short           ast_odbc_query_eof       ( void* query );

/* Moves the cursor position within the record set generated by SELECT query.
 * Returns "" in case of success and non empty human readable error text in case
 * of failure. This string is only valid until the next call to ast_odbc_* functions.
 * Do not call free() on it.
 * */
const char*     ast_odbc_query_fetch     ( void* query );

/* Returns
 *
 *       strlen(ast_odbc_query_fetch(query))!=0 ? 0 : !ast_odbc_query_eof();
 *
 * Very handy in quick record set enumerations without much of error checking
 * */
short           ast_odbc_query_next      ( void* query );

/* Returns the value of a named field in the record set in case of success.
 * Otherwise returns the default value passed in the last argument. If the
 * database type of the field is not a string, then the function converts
 * database value to string before the return.
 *
 * The lookup of field ordinal by the name is done with a binary search in
 * an array and it is very fast.
 *
 * The resulting pointer is only valid until the next call to ast_odbc_*
 * functions. Do not call free() on this value
 * */
const char*     ast_odbc_query_getstring ( void* query, const char* fname, const char* defaultValue );

/* These three functions return the value of the named field in the currect record
 * in case of success. In case of failure they return the default value given by
 * the last argument. If the database type is not the requested one, then functions
 * convert the database value to the requested type.
 * */
int             ast_odbc_query_getint    ( void* query, const char* fname, int defaultValue );
short           ast_odbc_query_getbool   ( void* query, const char* fname, short defaultValue );
time_t          ast_odbc_query_gettime   ( void* query, const char* fname, time_t defaultValue );

/* Returns text of the last error that happened within this query
 * This error text is valid only until the next call to ast_odbc_* functions.
 * Do not call free() on the result.
 * */
const char*     ast_odbc_query_error_text( void* query );

/* Returns !=0 is the specified field is present in the result set returned by the
 * query.
 * */
short           ast_odbc_query_got_column( void* query, const char* fname );

/* Destroys the query
 * Returns NULL in case of success. Otherwise returns its argument
 * */
void*           ast_odbc_query_delete    ( void* query );

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif  /* __ASTODBC_H__ */
