#ifndef __STRING_H__
#define __STRING_H__

#include <time.h>
#include <stdarg.h>

////////////////////////////////////////////////////////////////
// struct CStringData
////////////////////////////////////////////////////////////////
struct CStringData
{
        long nRefs;             // reference count
        int nDataLength;        // length of data (including terminator)
        int nAllocLength;       // length of allocation
        // char data[nAllocLength]
        char* data()           // char* to managed data
                { return (char*)(this+1); }
};

////////////////////////////////////////////////////////////////
// class CString
////////////////////////////////////////////////////////////////
class CString
{
public:
        // Constructors
        CString();                                              // constructs empty CString
        CString(const CString& stringSrc);                      // copy constructor
        CString(const char* lpsz);                              // from a character string
        CString(const char* lpsz, unsigned length );            // from a character string with lengt

        // Attributes & Operations
        int     GetLength() const;                              // get data length
        int     IsEmpty() const;                                // TRUE if zero length
        int     IsNotEmpty() const;                             // FALSE if zero length
        void    Empty();                                        // clear contents to empty

        char    GetAt(int nIndex) const;                        // return single character at zero-based index
        char    operator[](int nIndex) const;                   // return single character at zero-based index
        void    SetAt(int nIndex, char ch);                     // set a single character at zero-based index
        int     Compare(const char* lpsz) const;                // straight character comparison
        int     CompareNoCase(const char* lpsz) const;          // compare ignoring case
        operator const char*() const;                           // return pointer to const string
        CString Trim();                                         // Get a trimmed copy
        CString Replace( const char* pattern, const char* replacement, bool casesens=false, int howmany=-1 ) const;

        // overloaded assignment
        const CString& operator=(const CString& stringSrc);     // ref-counted copy from another CString
        const CString& operator=(char ch);                      // set string content to single character
        const CString& operator=(const char* lpsz);             // copy string content from ANSI string (converts to char)
        const CString& operator=(const unsigned char* psz);     // copy string content from unsigned chars

        // string concatenation
        const CString& operator+=(const char* lpsz);            // concatenate from const char*
        const CString& operator+=(const CString& string);       // concatenate from another CString
        const CString& operator+=(char ch);                     // concatenate a single character

        friend CString operator+(const CString& string1, const CString& string2);
        friend CString operator+(const CString& string, char ch);
        friend CString operator+(char ch, const CString& string);
        friend CString operator+(const CString& string, const char* lpsz);
        friend CString operator+(const char* lpsz, const CString& string);
        friend CString formatString( const char* format, ... );
        friend CString vformatString( const char* format, va_list ap );

// Implementation
public:
        ~CString();
        int GetAllocLength() const;

protected:
        char* m_pchData;   // pointer to ref counted string data

        // implementation helpers
        CStringData* GetData() const;
        void Init();
        void AllocCopy(CString& dest, int nCopyLen, int nCopyIndex, int nExtraLen) const;
        void AllocBuffer(int nLen);
        void AssignCopy(int nSrcLen, const char* lpszSrcData);
        void ConcatCopy(int nSrc1Len, const char* lpszSrc1Data, int nSrc2Len, const char* lpszSrc2Data);
        void ConcatInPlace(int nSrcLen, const char* lpszSrcData);
        void CopyBeforeWrite();
        void AllocBeforeWrite(int nLen);
        void Release();
        static void Release(CStringData* pData);
        static int  SafeStrlen(const char* lpsz);
        static void FreeData(CStringData* pData);
};

// Compare helpers
int      operator==(const CString& s1, const CString& s2);
int      operator==(const CString& s1, const char* s2);
int      operator==(const char* s1, const CString& s2);
int      operator!=(const CString& s1, const CString& s2);
int      operator!=(const CString& s1, const char* s2);
int      operator!=(const char* s1, const CString& s2);
int      operator<(const CString& s1, const CString& s2);
int      operator<(const CString& s1, const char* s2);
int      operator<(const char* s1, const CString& s2);
int      operator>(const CString& s1, const CString& s2);
int      operator>(const CString& s1, const char* s2);
int      operator>(const char* s1, const CString& s2);
int      operator<=(const CString& s1, const CString& s2);
int      operator<=(const CString& s1, const char* s2);
int      operator<=(const char* s1, const CString& s2);
int      operator>=(const CString& s1, const CString& s2);
int      operator>=(const CString& s1, const char* s2);
int      operator>=(const char* s1, const CString& s2);

// Other string functions
bool    compare_chars_case      ( char c1, char c2 );
bool    compare_chars_nocase    ( char c1, char c2 );
char*   searchSubstring         ( const char* haystack, const char* needle, bool (*comparer)(char,char) );
CString replaceSubstring        ( const char* s, const char* pattern, const char* replacement, bool casesens=false, int howmany=-1 );
CString vformatString           ( const char* format, va_list ap );
CString formatString            ( const char* format, ... );
CString formatTime              ( time_t time, const char* format="%Y-%m-%d %H:%M:%S" );
CString sqlEscapeString         ( const char* s );
CString quoteSqlString          ( const char* s );

#endif // __STRING_H__
