#ifndef __VECTOR_H__
#define __VECTOR_H__

#include <string.h>
#include <stdlib.h>

////////////////////////////////////////////////////////////////////////////////////////////
// class Vector<T> - implementation of dynamically growing array of T*
////////////////////////////////////////////////////////////////////////////////////////////
template<class T,const int BLOCK_SIZE=10> class Vector {
protected:
	T** m_buffer;
	int m_bufferSize;
	int m_size;

public:
	// methods
	int size() const
	{
		return m_size;
	}
	int add( T* element, int pos=-1 )
	{
		if( pos<0 || pos>=m_size )
		{
			pos = m_size; // add to the end
		}
		if( m_size>=m_bufferSize )
		{
			T** buffer = (T**)realloc(m_buffer,sizeof(T*)*(m_bufferSize+BLOCK_SIZE));
			if( !buffer ) return -1;
			m_buffer      = buffer;
			m_bufferSize += BLOCK_SIZE;
		}
		if( m_size>pos )
		{
			memmove(m_buffer+pos+1,m_buffer+pos,sizeof(T*)*(m_size-pos));
		}
		m_buffer[pos] = element;
		m_size++;
		return m_size;
	}
	T* remove( int pos ) {
		if( pos<0 || pos>=m_size )
		{
			return 0;
		}
		T* result = m_buffer[pos];
		memmove(m_buffer+pos,m_buffer+pos+1,sizeof(T*)*(m_size-pos-1));
		m_size--;
		if( m_bufferSize-m_size>=BLOCK_SIZE )
		{
			if( m_bufferSize==BLOCK_SIZE )
			{
				free(m_buffer);
				m_buffer = 0;
			}
			else
			{
				m_buffer = (T**)realloc(m_buffer,sizeof(T*)*(m_bufferSize-BLOCK_SIZE));
			}
			m_bufferSize -= BLOCK_SIZE;
		}
		return result;
	}
	int find( const T* el ) const {
		for( int n=0; n<m_size; n++ )
		{
			if( m_buffer[n]==el )
			{
				return n;
			}
		}
		return -1;
	}
	int bsearch_position( const void* key, int (*comparer)(const void* key,const T*), bool* found=NULL ) const {
		// Standard bsearch function does not work here because there is need to figure out
		// the location of a key in the vector even if jey is not present yet
		int left  = 0;
		int right = size();
		if( found )
		*found = false;
		while( left<right ) {
			int position = (left+right)/2;
			int result   = comparer(key,get(position));
			if( result<0 ) {
				right = position;
			}
			else
				if( result==0 ) {
					if( found )
						*found = true;
					return position;
				}
				else
					if( left<position ) {
						left = position;
					}
					else {
						return right;
					}
		}
		return left;
	}
	T* bsearch( const void* key, int (*comparer)(const void* key,const T*) ) const {
		bool found;
		int n = bsearch_position(key,comparer,&found);
		return found ? m_buffer[n] : NULL;
	}
	T* lsearch( const void* key, int (*comparer)(const void* key,const T*) ) const {
		for( int n=0; n<m_size; n++ ) {
			if( comparer(key,m_buffer[n])==0 )
			return m_buffer[n];
		}
		return NULL;
	}
	void flush() {
		// start flushing from the end so that other threads accessing
		// the object have more accurate idea of what the current size
		// of the vector is. This is not a perfect multithread solution
		// but better than nothing.
		while( 0<m_size-- )
		{
			delete m_buffer[m_size];
			m_buffer[m_size] = 0;
		}
		free(m_buffer);
		m_buffer = 0;
		m_bufferSize = 0;
		m_size = 0;
	}
	Vector<T>& operator=( const Vector<T>& l ) {
		flush();
		if( l.m_bufferSize>0 )
		{
			m_buffer = (T**)malloc(sizeof(T*)*l.m_bufferSize);
			for( int n=0; n<l.m_size; n++ )
			{
				m_buffer[n] = new T(*l.m_buffer[n]);
			}
			m_bufferSize = l.m_bufferSize;
			m_size = l.m_size;
		}
		return *this;
	}
	T* get( int pos ) const {
		if( pos<0 || pos>=m_size )
		{
			return 0;
		}
		return m_buffer[pos];
	}
	T* remove_el( const T* element ) {
		return remove(find(element));
	}
	void remove_all() {
		free(m_buffer);
		m_buffer = 0;
		m_bufferSize = 0;
		m_size = 0;
	}
	operator T** () {
		return m_buffer;
	}
	// constructor
	Vector() :
	m_buffer(0),
	m_bufferSize(0),
	m_size(0) {
	}
	~Vector() {
		flush();
	}
};

////////////////////////////////////////////////////////////////////////////////////////////
// class StaticVector<T> - implementation of array of T* with statically given max size
////////////////////////////////////////////////////////////////////////////////////////////
template<class T,const int MAX_SIZE=256> class StaticVector {
public:
	// properties
	T*      m_buff[MAX_SIZE];
	int     m_size;
	// methods
	bool got_room() {
		return m_size<MAX_SIZE;
	}
	T* push( T* t ) {
		if( !got_room() ) return NULL;
		m_buff[m_size++] = t;
		return t;
	}
	T* pop() {
		return m_buff[--m_size];
	}
	void flush() {
		m_size = 0;
	}
	StaticVector() {
		memset(m_buff,0,sizeof(m_buff));
		m_size = 0;
	}
};

#endif /* __VECTOR_H__ */
